==> pdapi-1.20.1+0.3.9-sources.jar: ml/pluto7073/pdapi/addition/DrinkAdditionManager.java <==
package ml.pluto7073.pdapi.addition;

import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.mojang.serialization.JsonOps;
import ml.pluto7073.chemicals.Chemicals;
import ml.pluto7073.pdapi.PDAPI;
import ml.pluto7073.pdapi.PDRegistries;
import ml.pluto7073.pdapi.addition.action.OnDrinkAction;
import ml.pluto7073.pdapi.addition.action.OnDrinkSerializer;
import ml.pluto7073.pdapi.networking.packet.clientbound.ClientboundSyncAdditionRegistryPacket;
import ml.pluto7073.pdapi.util.DrinkUtil;
import net.fabricmc.fabric.api.event.lifecycle.v1.ServerLifecycleEvents;
import net.fabricmc.fabric.api.networking.v1.ServerPlayNetworking;
import net.fabricmc.fabric.api.resource.SimpleSynchronousResourceReloadListener;
import net.fabricmc.fabric.api.resource.conditions.v1.ResourceConditions;
import net.minecraft.class_2960;
import net.minecraft.class_3222;
import net.minecraft.class_3298;
import net.minecraft.class_3300;
import net.minecraft.class_3518;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.*;

public class DrinkAdditionManager implements SimpleSynchronousResourceReloadListener {

    private static final Map<class_2960, DrinkAddition> REGISTRY = new HashMap<>();
    private static final Map<class_2960, DrinkAddition> STATIC_REGISTRY = new HashMap<>();

    public static final String ADDITIONS_NBT_KEY = "Additions";
    public static final DrinkAddition EMPTY = register(PDAPI.asId("empty"), new DrinkAddition.Builder().build());
    public static final class_2960 PHASE = PDAPI.asId("phase/additions");

    public DrinkAdditionManager() {
        ServerLifecycleEvents.SYNC_DATA_PACK_CONTENTS.register(PHASE, (player, joined) -> send(player));
    }

    public static DrinkAddition register(class_2960 id, DrinkAddition addition) {
        return register(id, addition, true);
    }

    public static DrinkAddition register(class_2960 id, DrinkAddition addition, boolean staticAdd) {
        if (containsId(id)) {
            if (get(id).getCurrentWeight() >= addition.getCurrentWeight()) return get(id);
        }
        REGISTRY.put(id, addition);
        if (staticAdd) STATIC_REGISTRY.put(id, addition);
        return addition;
    }

    public static class_2960 getId(DrinkAddition addition) {
        for (Map.Entry<class_2960, DrinkAddition> entry : REGISTRY.entrySet()) {
            if (Objects.equals(entry.getValue(), addition)) {
                return entry.getKey();
            }
        }
        throw new IllegalArgumentException("Unregistered drink addition: " + addition.toString());
    }

    public static DrinkAddition get(class_2960 id) {
        return REGISTRY.get(id);
    }

    public static void resetRegistry() {
        REGISTRY.clear();
        REGISTRY.putAll(STATIC_REGISTRY);
    }

    public static boolean containsId(class_2960 id) {
        return REGISTRY.containsKey(id);
    }

    public static boolean containsAddition(DrinkAddition addition) {
        return REGISTRY.containsValue(addition);
    }

    public static boolean contains(class_2960 id, DrinkAddition addition) {
        return containsId(id) && containsAddition(addition) && get(id).equals(addition);
    }

    public static boolean contains(Map.Entry<class_2960, DrinkAddition> entry) {
        return contains(entry.getKey(), entry.getValue());
    }

    public static void send(class_3222 entity) {

        ServerPlayNetworking.send(entity, new ClientboundSyncAdditionRegistryPacket(REGISTRY));

    }

    @Override
    public class_2960 getFabricId() {
        return PDAPI.asId("drink_addition_registerer");
    }

    @Override
    public void method_14491(class_3300 manager) {
        resetRegistry();

        int i = 0;

        for (Map.Entry<class_2960, class_3298> entry : manager.method_14488("drink_additions", id -> id.method_12832().endsWith(".json")).entrySet()) {
            class_2960 id = DrinkUtil.getAsId(entry.getKey(), "drink_additions");
            try (InputStream stream = entry.getValue().method_14482()) {
                JsonObject object = class_3518.method_15255(new InputStreamReader(stream));

                if (object.has("fabric:load_conditions")) {
                    boolean b = ResourceConditions.conditionsMatch(
                            class_3518.method_15261(object, "fabric:load_conditions"),
                            true
                    );

                    if (!b) continue;
                }

                register(id, DrinkAddition.CODEC.parse(JsonOps.INSTANCE, object).getOrThrow(false, s -> {
                    throw new IllegalStateException(s);
                }), false);
                i++;
            } catch (IOException e) {
                PDAPI.LOGGER.error("Could not load Drink Addition {}", id, e);
            }
        }

        PDAPI.LOGGER.info("Loaded {} additions", i);
    }

    @Override
    public ArrayList<class_2960> getFabricDependencies() {
        return new ArrayList<>();
    }

}

==> pdapi-1.20.4+1.0.1-sources.jar: ml/pluto7073/pdapi/addition/DrinkAdditionManager.java <==
package ml.pluto7073.pdapi.addition;

import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParseException;
import com.mojang.serialization.JsonOps;
import ml.pluto7073.pdapi.PDAPI;
import ml.pluto7073.pdapi.PDRegistries;
import ml.pluto7073.pdapi.addition.action.OnDrinkAction;
import ml.pluto7073.pdapi.addition.action.OnDrinkSerializer;
import ml.pluto7073.pdapi.addition.chemicals.ConsumableChemicalRegistry;
import ml.pluto7073.pdapi.networking.packet.clientbound.ClientboundSyncAdditionRegistryPacket;
import ml.pluto7073.pdapi.util.DrinkUtil;
import net.fabricmc.fabric.api.event.lifecycle.v1.ServerLifecycleEvents;
import net.fabricmc.fabric.api.networking.v1.ServerPlayNetworking;
import net.fabricmc.fabric.api.resource.SimpleSynchronousResourceReloadListener;
import net.fabricmc.fabric.api.resource.conditions.v1.ResourceConditions;
import net.minecraft.class_156;
import net.minecraft.class_2960;
import net.minecraft.class_3222;
import net.minecraft.class_3298;
import net.minecraft.class_3300;
import net.minecraft.class_3518;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.*;

public class DrinkAdditionManager implements SimpleSynchronousResourceReloadListener {

    private static final Map<class_2960, DrinkAddition> REGISTRY = new HashMap<>();
    private static final Map<class_2960, DrinkAddition> STATIC_REGISTRY = new HashMap<>();

    public static final String ADDITIONS_NBT_KEY = "Additions";
    public static final DrinkAddition EMPTY = register(PDAPI.asId("empty"), new DrinkAddition.Builder().build());
    public static final class_2960 PHASE = PDAPI.asId("phase/additions");

    public DrinkAdditionManager() {
        ServerLifecycleEvents.SYNC_DATA_PACK_CONTENTS.register(PHASE, (player, joined) -> send(player));
    }

    public static DrinkAddition register(class_2960 id, DrinkAddition addition) {
        return register(id, addition, true);
    }

    public static DrinkAddition register(class_2960 id, DrinkAddition addition, boolean staticAdd) {
        if (containsId(id)) {
            if (get(id).getCurrentWeight() >= addition.getCurrentWeight()) return get(id);
        }
        REGISTRY.put(id, addition);
        if (staticAdd) STATIC_REGISTRY.put(id, addition);
        return addition;
    }

    public static class_2960 getId(DrinkAddition addition) {
        for (Map.Entry<class_2960, DrinkAddition> entry : REGISTRY.entrySet()) {
            if (addition.equals(entry.getValue())) {
                return entry.getKey();
            }
        }
        throw new IllegalArgumentException("Unregistered drink addition: " + addition.toString());
    }

    public static DrinkAddition get(class_2960 id) {
        return REGISTRY.get(id);
    }

    public static void resetRegistry() {
        REGISTRY.clear();
        REGISTRY.putAll(STATIC_REGISTRY);
    }

    public static boolean containsId(class_2960 id) {
        return REGISTRY.containsKey(id);
    }

    public static boolean containsAddition(DrinkAddition addition) {
        return REGISTRY.containsValue(addition);
    }

    public static boolean contains(class_2960 id, DrinkAddition addition) {
        return containsId(id) && containsAddition(addition) && get(id).equals(addition);
    }

    public static boolean contains(Map.Entry<class_2960, DrinkAddition> entry) {
        return contains(entry.getKey(), entry.getValue());
    }

    public static void send(class_3222 entity) {

        ServerPlayNetworking.send(entity, new ClientboundSyncAdditionRegistryPacket(REGISTRY));

    }

    @Override
    public class_2960 getFabricId() {
        return PDAPI.asId("drink_addition_registerer");
    }

    @Override
    public void method_14491(class_3300 manager) {
        resetRegistry();

        int i = 0;

        for (Map.Entry<class_2960, class_3298> entry : manager.method_14488("drink_additions", id -> id.method_12832().endsWith(".json")).entrySet()) {
            class_2960 id = DrinkUtil.getAsId(entry.getKey(), "drink_additions");
            try (InputStream stream = entry.getValue().method_14482()) {
                JsonObject object = class_3518.method_15255(new InputStreamReader(stream));

                if (object.has("fabric:load_conditions")) {
                    boolean b = ResourceConditions.conditionsMatch(
                            class_3518.method_15261(object, "fabric:load_conditions"),
                            true
                    );

                    if (!b) continue;
                }

                register(id, loadFromJson(id, object), false);
                i++;
            } catch (IOException e) {
                PDAPI.LOGGER.error("Could not load Drink Addition {}", id, e);
            }
        }

        PDAPI.LOGGER.info("Loaded {} additions", i);
    }

    @Override
    public ArrayList<class_2960> getFabricDependencies() {
        return new ArrayList<>();
    }

    public static DrinkAddition loadFromJson(class_2960 id, JsonObject object) {
        return class_156.method_47526(DrinkAddition.CODEC.parse(JsonOps.INSTANCE, object), JsonParseException::new);
    }

}
